// ZipMaxDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ZipMax.h"
#include "ZipMaxDlg.h"
#include "misc.h"
#include "zipstructs.h"
#include <algorithm>

#define DATABUFFERSIZE 8*1024*1024	// buffersize for WriteLFH(...) 8 MB


//////////////////////////////////////////////////////////////////////////


CZipEntry::CZipEntry()
{
	// standard init...

	m_pHeader = NULL;
	m_strEntryName = "";
	m_strArchiveName = "";
}


bool CZipEntry::operator < (const CZipEntry& entry) const
{
	// operator used for stable_sort....
	// sort m_strEntryName

	return (m_strEntryName.CompareNoCase(entry.m_strEntryName) < 0);
}


//////////////////////////////////////////////////////////////////////////



// CZipMaxDlg dialog



CZipMaxDlg::CZipMaxDlg(CWnd* pParent /*=NULL*/)	: CDialog(CZipMaxDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	// initialize the members....

	ZeroMemory(&m_param, sizeof (THREADPARAM));

	m_vecFiles.clear();
	m_pListCtrl = NULL;
	m_pStatus1 = NULL;
	m_pStatus2 = NULL;
	m_pThread = NULL;
	m_pProgCtrl = NULL;
	m_pButtonPause = NULL;
}

void CZipMaxDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	return;
}

BEGIN_MESSAGE_MAP(CZipMaxDlg, CDialog)
	//{{AFX_MSG_MAP(CZipMaxDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DROPFILES()
	ON_WM_SIZE()
	ON_NOTIFY(LVN_COLUMNCLICK, IDC_LIST, OnColumnClick)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_PAUSE, OnBnClickedButtonPause)
END_MESSAGE_MAP()


// CZipMaxDlg message handlers

BOOL CZipMaxDlg::OnInitDialog()
{
	CString strTitle;
	strTitle.LoadString(IDS_STRING_TITLE);
	SetWindowText(strTitle);

	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);


	// get pointer to controls

	m_pProgCtrl = (CProgressCtrl *)GetDlgItem(IDC_PROGRESS);
	m_pListCtrl = (CListCtrl *)GetDlgItem(IDC_LIST);
	m_pStatus1 = (CStatic *)GetDlgItem(IDC_STATUS1);
	m_pStatus2 = (CStatic *)GetDlgItem(IDC_STATUS2);
	m_pButtonPause = (CButton *)GetDlgItem(IDC_BUTTON_PAUSE);


	// set statuslines

	CString q;
	q.LoadString(IDS_DROPFILES);
	m_pStatus1->SetWindowText(q);

	UpdateStatus(m_pStatus2);


	// prepare listcontrol, add 3 columns

	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();

	LVCOLUMN column;
	::ZeroMemory(&column, sizeof(LVCOLUMN));
	column.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT;

	CString str;

	GetColumnInfo(0,&column,str);
	column.cx = pApp->g_nWidth0;
	m_pListCtrl->InsertColumn(0,&column);

	GetColumnInfo(1,&column,str);
	column.cx = pApp->g_nWidth1;
	m_pListCtrl->InsertColumn(1,&column);

	GetColumnInfo(2,&column,str);
	column.cx = pApp->g_nWidth2;
	m_pListCtrl->InsertColumn(2,&column);

	m_pListCtrl->SetExtendedStyle(LVS_EX_FULLROWSELECT);

	m_bmStatus[0].LoadBitmap(IDB_STATUS_GREEN);
	m_bmStatus[1].LoadBitmap(IDB_STATUS_YELLOW);
	m_bmStatus[2].LoadBitmap(IDB_STATUS_RED);
	m_bmArrowUp.LoadBitmap(IDB_ARROW_UP);
	m_bmArrowDown.LoadBitmap(IDB_ARROW_DOWN);
	m_ImageList.Create(14,14,ILC_MASK,5,0);
	m_ImageList.Add(&m_bmStatus[0],RGB(192,192,192));
	m_ImageList.Add(&m_bmStatus[1],RGB(192,192,192));
	m_ImageList.Add(&m_bmStatus[2],RGB(192,192,192));
	m_ImageList.Add(&m_bmArrowUp,RGB(192,192,192));
	m_ImageList.Add(&m_bmArrowDown,RGB(192,192,192));
	m_ImageList.SetBkColor(CLR_NONE);
	m_pListCtrl->SetImageList(&m_ImageList,LVSIL_SMALL);
	m_nLastColumn = -1;

	// resize/move window to stored position

	MoveWindow(pApp->g_rectOutput);


	// get command line filenames (vector is cleared in GetAllCmdLineZip())

	CMisc::GetAllCmdLineZip(pApp->m_lpCmdLine, &m_vecFiles, pApp->g_bSubfolders);

	if ((DWORD)m_vecFiles.size() > 0)
		StartZipping ();

	return TRUE;
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CZipMaxDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
		CDialog::OnPaint();
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.

HCURSOR CZipMaxDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



void CZipMaxDlg::OnDropFiles(HDROP hDropInfo)
{
	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();


	// don't accept dropped files when already processing some

	if (pApp->g_bThreadRunning)
	{
		CDialog::OnDropFiles(hDropInfo);
		return;
	}

	// Remove sort arrows (if any), because this new job will not have anything sorted.
	CString str;
	LVCOLUMN column;
	if (m_nLastColumn >= 0)
	{
		GetColumnInfo(m_nLastColumn,&column,str);
		column.mask = LVCF_FMT;
		m_pListCtrl->SetColumn(m_nLastColumn,&column);
		m_nLastColumn = -1;
	}

	// get dropped filenames (vector is cleared in GetAllDroppedZip())

	CMisc::GetAllDroppedZip(hDropInfo, &m_vecFiles, pApp->g_bSubfolders);

	CDialog::OnDropFiles(hDropInfo);


	// start zipping

	StartZipping ();

	return;
}


void CZipMaxDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);

	CListCtrl *pListCtrl = (CListCtrl *)GetDlgItem(IDC_LIST);
	CProgressCtrl *pProgCtrl = (CProgressCtrl *)GetDlgItem(IDC_PROGRESS);
	CStatic *pStatus1 = (CStatic *)GetDlgItem(IDC_STATUS1);
	CStatic *pStatus2 = (CStatic *)GetDlgItem(IDC_STATUS2);
	CButton *pButtonPause = (CButton *)GetDlgItem(IDC_BUTTON_PAUSE);

	// resize controls to current windowsize

	if (pListCtrl != NULL && ::IsWindow(pListCtrl->m_hWnd))
	{
		CRect rectList, rectProgress, rectStatus1, rectStatus2, rectButtonPause;

		GetClientRect(&rectList);
		GetClientRect(&rectProgress);
		GetClientRect(&rectStatus1);
		GetClientRect(&rectStatus2);
		GetClientRect(&rectButtonPause);

		rectList.bottom -= 48;
		rectProgress.top = rectList.bottom;
		rectProgress.bottom = rectProgress.top + 16;
		rectProgress.right = rectStatus1.right - 16;
		rectStatus1.top = rectProgress.bottom;
		rectStatus1.bottom = rectStatus1.top + 16;
		rectStatus2.top = rectStatus1.bottom;

		rectButtonPause.top = rectProgress.top;
		rectButtonPause.bottom = rectProgress.bottom;
		rectButtonPause.left = rectProgress.right;
		rectButtonPause.right = rectButtonPause.left + 16;

		pListCtrl->MoveWindow(rectList);
		pProgCtrl->MoveWindow(rectProgress);
		pStatus1->MoveWindow(rectStatus1);
		pStatus2->MoveWindow(rectStatus2);
		pButtonPause->MoveWindow(rectButtonPause);

		pListCtrl->Invalidate();
		pProgCtrl->Invalidate();
		pStatus1->Invalidate();
		pStatus2->Invalidate();
		pButtonPause->Invalidate();
	}

	return;
}


void CZipMaxDlg::OnCancel()
{
	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();
	CListCtrl *pListCtrl = (CListCtrl *)GetDlgItem(IDC_LIST);

	CString p, q, q1, r;
	p.LoadString(IDS_STOPQUESTION);
	q.LoadString(IDS_QUIT);
	r.LoadString(IDS_PAUSEACTIVE);

	if (pApp->g_bPause)
	{
		MessageBox(r, q, MB_ICONWARNING | MB_OK);
		return;
	}

	if (pApp->g_bThreadRunning)
	{

		// pause thread

		if (m_pThread != NULL)
			m_pThread->SuspendThread();

		// show messagebox, ask user if he really wants to quit

		if (MessageBox(p, q, MB_ICONQUESTION | MB_YESNO) == IDYES)
			pApp->g_bStop = TRUE;	// yes...so set the thread stop flag


		// resume thread

		if (m_pThread != NULL)
			m_pThread->ResumeThread();
	}
	else
	{
		// maximize window and store the current
		// position, size, column widths

		ShowWindow(SW_NORMAL);

		LVCOLUMN col;
		ZeroMemory(&col, sizeof(LVCOLUMN));
		col.mask = LVCF_WIDTH;
		pListCtrl->GetColumn(0, &col);
		pApp->g_nWidth0 = col.cx;
		ZeroMemory(&col, sizeof(LVCOLUMN));
		col.mask = LVCF_WIDTH;
		pListCtrl->GetColumn(1, &col);
		pApp->g_nWidth1 = col.cx;
		ZeroMemory(&col, sizeof(LVCOLUMN));
		col.mask = LVCF_WIDTH;
		pListCtrl->GetColumn(2, &col);
		pApp->g_nWidth2 = col.cx;

		GetWindowRect(&pApp->g_rectOutput);

		// quit

		CDialog::OnCancel();
	}

	return;
}



BOOL CZipMaxDlg::UnzipAllFiles(const CString& strArchiveName, const CString& strDestinationFolder, std::vector<CZipFileHeader> *pvecHeaders)
{
	// unzip files from a archive to a destinationfolder

	// returns: TRUE if unzip process was successfull, FALSE otherwise

	// const CString& strFileName       = full path/filename of the zipfile
	// const CString& strDestinationFolder    = full destinationfoldername
	// std::vector<CZipFileHeader> *pvecHeaders = point to a vector which is filled with headerinformation of the archive


	// change to the destinationfolder

	char cBuffer[_MAX_PATH];
	GetCurrentDirectory(_MAX_PATH, cBuffer);

	if (! SetCurrentDirectory(strDestinationFolder))
		return FALSE;


	// open the zipfile

	CZipArchive zip;
	bool bException = false;

	TRY
	{
		zip.Open(strArchiveName, CZipArchive::zipOpen || CZipArchive::zipOpenReadOnly);
	}
	CATCH(CException, pEx)
	{
		bException = true;
	}
	END_CATCH

	zip.SetAutoFlush(true);


	// don't work with spanned zips

	if (zip.GetSpanMode() != 0)
	{
		zip.Close();
		SetCurrentDirectory(cBuffer);
		return FALSE;
	}


	// extract all files

	bool bExtractOk = true;
	DWORD nIndex = 0;
	while (! bException && bExtractOk && nIndex < (DWORD)zip.GetCount())
	{
		TRY
		{
			bExtractOk &= zip.ExtractFile((WORD)nIndex++, NULL, true, NULL);
		}
		CATCH(CException, pEx)
		{
			bException = true;
		}
		END_CATCH
	}


	// close zipfile, change to old folder

	zip.Close(bException);
	_flushall();

	SetCurrentDirectory(cBuffer);


	// fill in the header information and quit

	return (((BOOL) ! bException) && (BOOL) bExtractOk) ? GetHeaders(strArchiveName, pvecHeaders) : FALSE;
}


BOOL CZipMaxDlg::InternalZipFiles(const CString& strArchiveName, const CString& strSourceFolder, std::vector<CZipFileHeader> *pvecHeaders)
{
	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();


	// skip Internal Zipping ?

	if (pApp->g_bSkipIntern)
		return TRUE;


	// change to sourcefolder (with the unzipped files)

	char cBuffer[_MAX_PATH];
	GetCurrentDirectory(_MAX_PATH, cBuffer);

	if (! SetCurrentDirectory(strSourceFolder))
		return FALSE;


	// get files from current folder

	std::vector<CString> vecFileList;
	CMisc::CountFilesInFolder(".\\", &vecFileList);

	bool bException = false;
	bool bAddOk = true;


	// create new zipfile and zip all files

	CZipArchive zip;

	TRY
	{
		zip.Open(strArchiveName, CZipArchive::zipCreate);
	}
	CATCH(CException, pEx)
	{
		bException = true;
	}
	END_CATCH

	zip.SetAutoFlush(true);


	if (! bException)
	{
		DWORD i = 0;
		while (bAddOk && ! bException && i < (DWORD)vecFileList.size())
		{
			CString str = CMisc::GetFullFileNameNoDrive(vecFileList[i]);

			if (vecFileList[i] != "" && str != "")
			{
				zip.SetRootPath(strSourceFolder);

				TRY
				{
					bAddOk &= zip.AddNewFile(vecFileList[i], Z_BEST_COMPRESSION, FALSE, CZipArchive::zipsmSmartest);
				}
				CATCH(CException, pEx)
				{
					bException = true;
				}
				END_CATCH
			}

			i++;
		}

		zip.Close(bException);
	}

	_flushall();

	DWORD n = (bAddOk && ! bException) ? 0 : 1;

	// Run the final command, if any.
	if ((n == 0) && (pApp->g_strFinalExe != ""))
	{
		CString strCommand = pApp->g_strFinalCommandLine;
		strCommand.Replace("%1", '\"' + strArchiveName + '\"');
		strCommand.Replace("%2", "*");
		n = CMisc::RunCmd(pApp->g_strFinalExe + " " + strCommand, GetThreadPriority(), pApp->g_bShowPackerWindow, &pApp->g_pi);
	}

	// get headerinformation from the new created zipfile

	const BOOL bReturn = (BOOL) (n == 0) ? GetHeaders(strArchiveName, pvecHeaders) : FALSE;

	// change to the old folder

	SetCurrentDirectory(cBuffer);

	return bReturn;
}


BOOL CZipMaxDlg::ExternalZipFiles(const CString& strArchiveName, const CString& strSourceFolder, std::vector<CZipFileHeader> *pvecHeaders, const DWORD nExternal)
{
	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();


	// change to sourcefolder

	char cBuffer[_MAX_PATH];
	GetCurrentDirectory(_MAX_PATH, cBuffer);

	if (! SetCurrentDirectory(strSourceFolder))
		return FALSE;


	// replace the commandline options with the current values

	CString strCommand = pApp->g_strCommandLine[nExternal];
	strCommand.Replace("%1", '\"' + strArchiveName + '\"');
	strCommand.Replace("%2", "*");


	// run the external packer

	DWORD n = CMisc::RunCmd(pApp->g_strPackerExe[nExternal] + " " + strCommand, GetThreadPriority(), pApp->g_bShowPackerWindow, &pApp->g_pi);

	// Run the final command, if any.
	if ((n == 0) && (pApp->g_strFinalExe != ""))
	{
		strCommand = pApp->g_strFinalCommandLine;
		strCommand.Replace("%1", '\"' + strArchiveName + '\"');
		strCommand.Replace("%2", "*");
		n = CMisc::RunCmd(pApp->g_strFinalExe + " " + strCommand, GetThreadPriority(), pApp->g_bShowPackerWindow, &pApp->g_pi);
	}


	// get headerinformation from the new created zipfile

	const BOOL bReturn = (BOOL) (n == 0) ? GetHeaders(strArchiveName, pvecHeaders) : FALSE;


	// change to old folder

	SetCurrentDirectory(cBuffer);

	return bReturn;
}


BOOL CZipMaxDlg::GetHeaders(const CString& strArchiveName, std::vector<CZipFileHeader> *pvecHeaders)
{

	// remove all listitems first

	pvecHeaders->clear();


	// open zipfile and get all headerinformation

	CZipArchive zip;
	bool bException = false;

	TRY
	{
		zip.Open(strArchiveName, CZipArchive::zipOpen || CZipArchive::zipOpenReadOnly);
	}
	CATCH(CException, pEx)
	{
		bException = true;
	}
	END_CATCH

	zip.SetAutoFlush(true);


	// don't work with spanned zips

	if (zip.GetSpanMode() != 0)
	{
		zip.Close();
		return FALSE;
	}


	// get the headers

	const DWORD nCount = zip.GetCount();
	pvecHeaders->reserve(nCount);

	CZipFileHeader info;
	bool bOk = TRUE;
	DWORD nIndex = 0;
	while (! bException && bOk && nIndex < nCount)
	{
		bOk &= zip.GetFileInfo(info, (WORD) nIndex++);

		if (bOk)
			pvecHeaders->push_back(info);
	}

	zip.Close(bException);

	return (BOOL) ((! bException) && bOk);
}


BOOL CZipMaxDlg::WriteLFH(CStdioFile *pFile, CZipEntry *pZipEntry, DWORD *pOffset, CString *pName)
{
	// allocate some memory for the compressed data

	BYTE *pDataBuffer = new BYTE[DATABUFFERSIZE];

	if (pDataBuffer == NULL)
		return FALSE;


	// fill localfileheader structure

	CZipStructs::ZIP_LFH lfh;

	const WORD nFileNameSize = pZipEntry->m_pHeader->GetFileNameSize();

	lfh.version_needed_to_extract = pZipEntry->m_pHeader->m_uVersionNeeded; // should match the value from cfh anyway
	lfh.general_purpose_bit_flag = pZipEntry->m_pHeader->m_uFlag & 0xfff7;  // remove possible datadescriptors
	lfh.compression_method = pZipEntry->m_pHeader->m_uMethod;       // should match the value from cfh anyway
	lfh.last_mod_file_time = pZipEntry->m_pHeader->m_uModTime;        // should match the value from cfh anyway
	lfh.last_mod_file_date = pZipEntry->m_pHeader->m_uModDate;        // should match the value from cfh anyway
	lfh.crc32 = pZipEntry->m_pHeader->m_uCrc32;               // use value from central dir because of the removal of datadescriptors
	lfh.compressed_size = pZipEntry->m_pHeader->m_uComprSize;       // use value from central dir because of the removal of datadescriptors
	lfh.uncompressed_size = pZipEntry->m_pHeader->m_uUncomprSize;     // use value from central dir because of the removal of datadescriptors
	lfh.filename_length = (WORD) nFileNameSize;                  // should match the value from cfh anyway
	lfh.extra_field_length = 0;                       // remove any extrafields

	BOOL bOk = TRUE;

	try
	{
		// write localfileheader and return the lfh offset

		*pOffset = pFile->GetPosition();

		const DWORD id = (DWORD) ID_PKZIPLFH;
		pFile->Write(&id, sizeof(DWORD));
		pFile->Write(&lfh, sizeof(CZipStructs::ZIP_LFH));
	}
	catch (CFileException *e)
	{
		e->Delete();
		bOk = FALSE;
	}

	if (bOk)
	{
		CStdioFile stioFile;
		CFileException e0;
		if (stioFile.Open(pZipEntry->m_strArchiveName, CFile::modeRead | CFile::typeBinary | CFile::shareExclusive, &e0))
		{
			try
			{
				// jump to lfh

				stioFile.Seek(pZipEntry->m_pHeader->m_uOffset + 4, CFile::begin);


				// read old lfh to get the real (!) extrafield length

				stioFile.Read(&lfh, sizeof(CZipStructs::ZIP_LFH));


				// get the stored filename (skip extrafield) and write it to the new file)

				stioFile.Read(pDataBuffer, nFileNameSize + lfh.extra_field_length);
				pFile->Write(pDataBuffer, nFileNameSize);


				// return the filename

				const CString s((char *)pDataBuffer, nFileNameSize);
				*pName = s;


				// write compressed data

				DWORD nCompressedSize = pZipEntry->m_pHeader->m_uComprSize;

				while (nCompressedSize != 0)
				{
					if (nCompressedSize < DATABUFFERSIZE)
					{
						stioFile.Read(pDataBuffer, nCompressedSize);
						pFile->Write(pDataBuffer, nCompressedSize);
						nCompressedSize = 0;
					}
					else
					{
						stioFile.Read(pDataBuffer, DATABUFFERSIZE);
						pFile->Write(pDataBuffer, DATABUFFERSIZE);
						nCompressedSize -= DATABUFFERSIZE;
					}
				}

				pFile->Flush();
				stioFile.Close();
			}
			catch (CFileException *e)
			{
				e->Delete();
				bOk = FALSE;
			}
		}
		else
		{
			bOk = FALSE;
		}
	}

	delete pDataBuffer;

	return bOk;
}


BOOL CZipMaxDlg::WriteCFH(CStdioFile *pFile, CZipEntry *pZipEntry, const DWORD nOffset, const CString& strEntryName)
{
	// fill central directory entry

	CZipStructs::ZIP_CFH cfh;

	cfh.version_made_by = pZipEntry->m_pHeader->m_uVersionMadeBy;
	cfh.version_needed_to_extract = pZipEntry->m_pHeader->m_uVersionNeeded;
	cfh.general_purpose_bit_flag = pZipEntry->m_pHeader->m_uFlag & 0xfff7;  // no datadescriptors
	cfh.compression_method = pZipEntry->m_pHeader->m_uMethod;
	cfh.last_mod_file_time = pZipEntry->m_pHeader->m_uModTime;
	cfh.last_mod_file_date = pZipEntry->m_pHeader->m_uModDate;
	cfh.crc32 = pZipEntry->m_pHeader->m_uCrc32;
	cfh.compressed_size = pZipEntry->m_pHeader->m_uComprSize;
	cfh.uncompressed_size = pZipEntry->m_pHeader->m_uUncomprSize;
	cfh.filename_length = pZipEntry->m_pHeader->GetFileNameSize();
	cfh.extra_field_length = 0;                       // no extrafields
	cfh.file_comment_length = 0;                      // no comments
	cfh.disk_number_start = 0;                        // no diskspanning zips
	cfh.internal_file_attributes = pZipEntry->m_pHeader->m_uInternalAttr;
	cfh.external_file_attributes = pZipEntry->m_pHeader->GetOriginalAttributes();
	cfh.relative_offset_of_local_header = nOffset;


	// write central directory entry

	BOOL bOk = TRUE;

	try
	{
		const DWORD id = ID_PKZIPCFH;
		pFile->Write(&id, sizeof(DWORD));
		pFile->Write(&cfh, sizeof(CZipStructs::ZIP_CFH));
		pFile->Write(strEntryName, cfh.filename_length);
		pFile->Flush();
	}
	catch (CFileException *e)
	{
		e->Delete();
		bOk = FALSE;
	}

	return bOk;
}



BOOL CZipMaxDlg::WriteEOZ(CStdioFile *pFile, const DWORD nCentralDirStart, const DWORD nCentralDirSize, const DWORD nEntries)
{

	// fill end-of-zip structure

	CZipStructs::ZIP_EOF eof;

	eof.end_of_central_dir_signature = (DWORD)ID_PKZIPEND;        // id
	eof.number_of_this_disk = 0;                    // no diskspanning
	eof.number_of_the_disk_with_the_start_of_the_central_directory = 0; // no diskspanning
	eof.total_number_of_entries_in_the_central_dir_on_this_disk = nEntries;
	eof.total_number_of_entries_in_the_central_dir = nEntries;
	eof.size_of_the_central_directory = nCentralDirSize;
	eof.offset_of_start_of_central_directory_with_respect_to_the_starting_disk_number = nCentralDirStart;
	eof.zipfile_comment_length = 0;                   // no comments

	BOOL bOk = TRUE;


	// write end-of-zip structure

	try
	{
		pFile->Write(&eof, sizeof(CZipStructs::ZIP_EOF));
		pFile->Flush();
	}
	catch (CFileException *e)
	{
		e->Delete();
		bOk = FALSE;
	}

	return bOk;
}



BOOL CZipMaxDlg::BuildNewZip(const CString& strArchiveName, std::vector<CZipEntry> *pvecEntries)
{
	BOOL bOk = TRUE;

	CStdioFile stdioFile;
	CFileException e0;

	const DWORD nCount = (DWORD)pvecEntries->size();

	std::vector<DWORD> vecLFHOffsets;
	std::vector<CString> vecEntryNames;
	vecLFHOffsets.reserve(nCount);
	vecEntryNames.reserve(nCount);

	if (stdioFile.Open(strArchiveName, CFile::modeWrite | CFile::modeCreate | CFile::typeBinary | CFile::shareExclusive, &e0))
	{
		try
		{
			for (DWORD i = 0; i < nCount; i++)
			{

				// write all localfileheaders, remember offsets & names

				CString strName;
				DWORD nOffset = 0;

				if (WriteLFH(&stdioFile, &((*pvecEntries)[i]), &nOffset, &strName))
				{
					vecLFHOffsets.push_back(nOffset);
					vecEntryNames.push_back(strName);
				}
				else
				{
					stdioFile.Flush();
					stdioFile.Close();
					CMisc::RemoveAFile(strArchiveName);
					return FALSE;
				}
			}


			// more or less offsets than entries ?
			// dummy check...shouldn't happen anyway

			if ((DWORD)vecLFHOffsets.size() != nCount && (DWORD)vecEntryNames.size() != nCount)
			{
				stdioFile.Flush();
				stdioFile.Close();
				CMisc::RemoveAFile(strArchiveName);
				return FALSE;
			}


			// remember start of the central dir

			const DWORD nStartOfCentralDir = stdioFile.GetPosition();


			// write all central directory entries

			for (i = 0; i < nCount; i++)
			{
				if (! WriteCFH(&stdioFile, &((*pvecEntries)[i]), vecLFHOffsets[i], vecEntryNames[i]))
				{
					stdioFile.Flush();
					stdioFile.Close();
					CMisc::RemoveAFile(strArchiveName);
					return FALSE;
				}
			}


			// write end-of-zipfile

			bOk &= WriteEOZ(&stdioFile, nStartOfCentralDir, stdioFile.GetPosition() - nStartOfCentralDir, nCount);

		}
		catch (CFileException *e)
		{
			e->Delete();
			bOk = FALSE;
		}

		stdioFile.Flush();
		stdioFile.Close();
		_flushall();
	}
	else
		bOk = FALSE;

	if (! bOk)
		CMisc::RemoveAFile(strArchiveName);		// if anything went wrong, remove the temporary file

	return bOk;
}


void CZipMaxDlg::UpdateList(CListCtrl *pListCtrl, const DWORD nIndex, const CString& s1, const CString& s2, const CString& s3)
{
	// add an item to the list

	LV_ITEM item;
	::ZeroMemory(&item, sizeof(LV_ITEM));

	int nStatusIndex = 0;

	if (s3 == "-")
		nStatusIndex = 2;

	else if
		(s3 == "0")
			nStatusIndex = 1;

	item.mask = LVIF_TEXT | LVIF_IMAGE;
	item.iImage = nStatusIndex;
	item.iItem = nIndex;
	item.lParam = 0;
	item.pszText = (LPTSTR)LPCTSTR(s1);
	item.cchTextMax = s1.GetLength();

	if (pListCtrl != NULL)
	{
		pListCtrl->InsertItem(&item);
		pListCtrl->SetItem(nIndex, 1, LVIF_TEXT, s2, 0, 0, 0, 0);
		pListCtrl->SetItem(nIndex, 2, LVIF_TEXT, s3, 0, 0, 0, 0);
		pListCtrl->EnsureVisible(nIndex, FALSE); // scroll to the new entry
	}

	return;
}


void CZipMaxDlg::WriteLog(CStdioFile *pFile, const CString& str)
{
	// write a line to the logfile

	if (pFile != NULL)
	{
		try
		{
			pFile->WriteString(str + "\n");
		}
		catch (CFileException *e)
		{
			e->Delete();
		}
	}

	return;
}


int CZipMaxDlg::GetThreadPriority()
{

	// returns the priority which was set by zipmax.ini

	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();

	int nPriority = THREAD_PRIORITY_NORMAL;

	if (pApp->g_strPriority.CompareNoCase("ABOVE_NORMAL") == 0)
		nPriority = THREAD_PRIORITY_ABOVE_NORMAL;

	if (pApp->g_strPriority.CompareNoCase("BELOW_NORMAL") == 0)
		nPriority = THREAD_PRIORITY_BELOW_NORMAL;

	if (pApp->g_strPriority.CompareNoCase("HIGHEST") == 0)
		nPriority = THREAD_PRIORITY_HIGHEST;

	if (pApp->g_strPriority.CompareNoCase("IDLE") == 0)
		nPriority = THREAD_PRIORITY_IDLE;

	if (pApp->g_strPriority.CompareNoCase("LOWEST") == 0)
		nPriority = THREAD_PRIORITY_LOWEST;

	if (pApp->g_strPriority.CompareNoCase("TIME_CRITICAL") == 0)
		nPriority = THREAD_PRIORITY_TIME_CRITICAL;

	return nPriority;
}


BOOL CZipMaxDlg::TestNewZip(const CString& strArchiveName)
{
	// opens an archive and tests each file
	// returns TRUE if all files are ok, FALSE otherwise
	// const CString& strArchiveName  = full archivename

	CZipArchive zip;
	bool bException = false;

	TRY
	{
		zip.Open(strArchiveName, CZipArchive::zipOpen || CZipArchive::zipOpenReadOnly);
	}
	CATCH(CException, pEx)
	{
		bException = true;
	}
	END_CATCH

	zip.SetAutoFlush(true);


	// test all files

	DWORD nIndex = 0;
	bool bTestOk = true;
	while (! bException && bTestOk && nIndex < (DWORD)zip.GetCount())
	{
		TRY
		{
			bTestOk &= zip.TestFile((WORD)nIndex++);
		}
		CATCH(CException, pEx)
		{
			bException = true;
		}
		END_CATCH
	}

	return ((BOOL)bTestOk && (BOOL)(! bException));
}


BOOL CZipMaxDlg::SafeRename(const CString& strSource)
{
	// a secure way to update the old file with the new one...
	// rename old to temp, new to old, remove temp....

	BOOL bOk = TRUE;

	CString strTmpExt, strTmpExt2;
	strTmpExt.LoadString(IDS_TEMPEXTENSION);
	strTmpExt2.LoadString(IDS_TEMPEXTENSION2);

	try
	{
		CFile::Rename(strSource, strSource + strTmpExt2);
	}
	catch (CFileException *e)
	{
		e->Delete();
		bOk = FALSE;
	}
	if (bOk)
	{
		try
		{
			CFile::Rename(strSource + strTmpExt, strSource);
		}
		catch (CFileException *e)
		{
			e->Delete();
			bOk = FALSE;
		}
	}
	if (bOk)
		CMisc::RemoveAFile(strSource + strTmpExt2);
	else
	{
		CMisc::RemoveAFile(strSource + strTmpExt);

		try
		{
			CFile::Rename(strSource + strTmpExt2, strSource);
		}
		catch (CFileException *e)
		{
			e->Delete();
		}
	}

	return bOk;
}


void CZipMaxDlg::UpdateStatus(CStatic *pStatus, const DWORD nIndex, const DWORD nTotalFiles, const DWORD nCurrentTotal, const DWORD nRemainingSecs, const DWORD nDoneSize)
{
	// set a status line with remaining files, est. time, etc...

	CString s;
	s.LoadString(IDS_SEPARATOR);

	CString strFiles, strPercent, strPercentSaved, strCurrentTotal, strTime, q;

	q.LoadString(IDS_REMAININGFILES);
	strFiles = q + CMisc::NumericToString(nTotalFiles - nIndex) + s;

	q.LoadString(IDS_DONEPERCENT);
	const double dPercent = 100.0 * (double) nIndex / (double) nTotalFiles;
	strPercent.Format(q, (nTotalFiles == 0) ? 0 : (DWORD) dPercent);
	strPercent += s;

	const DWORD nSeconds = nRemainingSecs % 60;
	const DWORD nMinutes = (nRemainingSecs / 60) % 60;
	const DWORD nHours = (nRemainingSecs / 3600);

	q.LoadString(IDS_REMAININGTIME);
	strTime.Format(q + s, nHours, nMinutes, nSeconds);

	q.LoadString(IDS_SAVEDBYTES);
	strCurrentTotal = q + CMisc::NumericToString(nCurrentTotal);

	q.LoadString(IDS_PERCENTSAVEDBYTES);
	const double dPercentSaved = (nDoneSize == 0) ? 0 : (100.0 * (double) nCurrentTotal / (double) nDoneSize);
	strPercentSaved.Format (q, dPercentSaved);

	pStatus->SetWindowText(strFiles + strPercent + strTime + strCurrentTotal + strPercentSaved);

	return;
}


UINT CZipMaxDlg::WorkThread(LPVOID pParam)
{
	// Main Zipping thread. Anything is done here :O)
	// I know it should be a lot smaller....more subroutines etc...


	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();

	// set thread flag

	pApp->g_bThreadRunning = TRUE;

	THREADPARAM *p = (THREADPARAM *)pParam;


	// prepare some foldernames

	CString t,strExternFolder[MAXEXTERNAL];
	t.LoadString(IDS_TEMPUNZIP);
	const CString strUnzipFolder = *(p->pstrMainFolder) + t;

	t.LoadString(IDS_TEMPEXTERN);
	for (DWORD x = 0; x < pApp->g_dwNumExtern; x++)
		strExternFolder[x].Format(*(p->pstrMainFolder) + t, x + 1);

	t.LoadString(IDS_TEMPINTERN);
	const CString strInternFolder = *(p->pstrMainFolder) + t;


	CString q, strError, strTmpExt; // for statustexts && logfile and tempextension
	strTmpExt.LoadString(IDS_TEMPEXTENSION);


	// prepare logfile

	CStdioFile *pFile = NULL;
	CStdioFile logFile;
	if (pApp->g_bLogFile)
	{
		CFileException e0;
		q.LoadString(IDS_LOGFILE);
		if (logFile.Open(*(p->pstrMainFolder) + q, CFile::modeWrite | CFile::modeCreate | CFile::modeNoTruncate | CFile::typeText | CFile::shareExclusive, &e0))
		{
			logFile.SeekToEnd();
			pFile = &logFile;
		}
	}


	// let's go

	DWORD nTotalGain = 0;

	UpdateStatus(p->pStatus2, 0, (DWORD)p->pvecFiles->size(), nTotalGain, p->nRemainingSize >> 16); // let's start with a default value of 1sec/64kbyte

	for (DWORD i = 0; i < (DWORD)p->pvecFiles->size(); i++)
	{
		if (pApp->g_bStop)
			break;

	    CString cMinStr;
		BOOL bFileListed = FALSE;

		std::vector<CZipFileHeader> vecHeadersOld;
		std::vector<CZipFileHeader> vecHeadersIntern;
		std::vector<CZipFileHeader> vecHeadersExtern[MAXEXTERNAL];

		CString strExternFile[MAXEXTERNAL];
		const CString strSource = (*p->pvecFiles)[i];
		const CString strInternFile = strInternFolder + CMisc::GetPureFileName(strSource);

		for (x = 0; x < pApp->g_dwNumExtern; x++)
			strExternFile[x] = strExternFolder[x] + CMisc::GetPureFileName(strSource);


		// cleanup + create folders

		CMisc::RemoveAFolder(strUnzipFolder);
		for (x = 0; x < pApp->g_dwNumExtern; x++)
			CMisc::RemoveAFolder(strExternFolder[x]);
		CMisc::RemoveAFolder(strInternFolder);

		CreateDirectory(strUnzipFolder, NULL);
		for (x = 0; x < pApp->g_dwNumExtern; x++)
			CreateDirectory(strExternFolder[x], NULL);
		CreateDirectory(strInternFolder, NULL);


		BOOL bFolderOk = TRUE;
		bFolderOk &= CMisc::IsFolderEmpty(strUnzipFolder);
		for (x = 0; x < pApp->g_dwNumExtern; x++)
			bFolderOk &= CMisc::IsFolderEmpty(strExternFolder[x]);
		bFolderOk &= CMisc::IsFolderEmpty(strInternFolder);


		UINT64 nCurrentSize = 0;
		CMisc::GetFileSize(strSource, &nCurrentSize);
		CMisc::GetTimeSpan();

		// -- START: Special file skipping code -- 
		BOOL bSkipFile = FALSE;
		CFileStatus rStatus;
		CFile::GetStatus(strSource, rStatus);

		// If requested, check if file is Archive, skip if it is
		if (pApp->g_bSkipArchive && (rStatus.m_attribute & CFile::archive)) {
			q.LoadString(IDS_SKIPPEDA);
			strError.LoadString(IDS_IDSARCHIVESET);
			bSkipFile = true;
		}
		// If requested, check if file is ReadOnly, skip if it is
		else {
			if (pApp->g_bSkipReadOnly && (rStatus.m_attribute & CFile::readOnly)) {
				q.LoadString(IDS_SKIPPEDR);
				strError.LoadString(IDS_IDSREADONLYSET);
				bSkipFile = true;
			}
			// If requested, mark file for ReadOnly (it is unchecked later on replacement) @@what if no savings?
			else {
				if (pApp->g_bSetCurrentReadOnly) {
					rStatus.m_attribute |= CFile::readOnly;
					try {
						CFile::SetStatus(strSource, rStatus);
					}
					catch (CFileException *e) {
						e->Delete();
					}
				}
			}
		}
		if (bSkipFile) {
			UpdateList(p->pListCtrl, i, strSource, q, "0");
			WriteLog(pFile, q + ": " + strSource + " -> " + strError + "\n");

			// Make it skip writing different 'real' error below (TODO: Restructure if statements/merge this in below)
			bFileListed = true;		
			bFolderOk = false;
		}
		// -- END: Special file skipping code -- 

		if (bFolderOk)
		{
			// unzip files

			q.LoadString(IDS_UNZIP);
			p->pStatus1->SetWindowText(q + CMisc::GetPureFileName(strSource));
			if (UnzipAllFiles(strSource, strUnzipFolder, &vecHeadersOld))
			{
				// internal zip

				if (pApp->g_bStop)
					break;

				q.LoadString(IDS_INTERNAL);
				p->pStatus1->SetWindowText(q + CMisc::GetPureFileName(strSource));
				if (InternalZipFiles(strInternFile, strUnzipFolder, &vecHeadersIntern))
				{
					// external zip

					BOOL bOK = TRUE;
					CString strStatus;
					q.LoadString(IDS_EXTERNAL);

					for (x = 0; x < pApp->g_dwNumExtern; x++)
					{
						if (pApp->g_bStop)
							break;

						strStatus.Format(q + CMisc::GetPureFileName(strSource), x + 1, pApp->g_dwNumExtern);
						p->pStatus1->SetWindowText(strStatus);

						if (! ExternalZipFiles(strExternFile[x], strUnzipFolder, &vecHeadersExtern[x], x))
						{
							bOK = FALSE;
							break;
						}
					}

					if (pApp->g_bStop)
						break;

					if (bOK)
					{
						// analyze files

						if (pApp->g_bStop)
							break;

						q.LoadString(IDS_ANALYZE);
						p->pStatus1->SetWindowText(q + CMisc::GetPureFileName(strSource));

						const DWORD nCount = (DWORD)vecHeadersOld.size();

						BOOL bError = FALSE;
						std::vector<CZipEntry> vecEntries;
						vecEntries.reserve(nCount);

						for (DWORD h = 0; h < nCount; h++)
						{
							const CString strName = vecHeadersOld[h].GetFileName();

							if (strName.Right(1) != '\\' && strName.Right(1) != '/') // skip pure subfolders
							{
								// search the currentfile in the external zips

								DWORD nIndexExternal[MAXEXTERNAL];

								for (x = 0; x < pApp->g_dwNumExtern; x++)
								{
									nIndexExternal[x] = 0;
									BOOL bFoundExternal = FALSE;

									while (! bFoundExternal && (nIndexExternal[x] != (DWORD)vecHeadersExtern[x].size()))
										bFoundExternal = (strName.CompareNoCase(vecHeadersExtern[x][nIndexExternal[x]++].GetFileName()) == 0);

									nIndexExternal[x]--;

									if (nIndexExternal[x] == -1 || ! bFoundExternal)
									{
										bError = TRUE;
										break;
									}
								}

								if (bError)
									break;

								// search the currentfile in the internal zip

								DWORD nIndexInternal = 0;
								BOOL bFoundInternal = FALSE;
								while (! bFoundInternal && (nIndexInternal < (DWORD)vecHeadersIntern.size()))
									bFoundInternal = (strName.CompareNoCase(vecHeadersIntern[nIndexInternal++].GetFileName()) == 0);

								nIndexInternal--;

								if (! pApp->g_bSkipIntern  && (nIndexInternal == -1 || ! bFoundInternal))
								{
									bError = TRUE;
									break;
								}


								// get the sizes

								const DWORD nOldSize = vecHeadersOld[h].m_uComprSize;
								const DWORD nInternSize = pApp->g_bSkipIntern ? nOldSize + 1 : vecHeadersIntern[nIndexInternal].m_uComprSize;

								DWORD nExternSize[MAXEXTERNAL];
								for (x = 0; x < pApp->g_dwNumExtern; x++)
									nExternSize[x] = vecHeadersExtern[x][nIndexExternal[x]].m_uComprSize;


								// compare the sizes and choose the smallest one

								CZipEntry entry;
								entry.m_pHeader = &vecHeadersOld[h];
								entry.m_strArchiveName = strSource;
								entry.m_strEntryName = strName;

								DWORD nMinSize = nOldSize;

								if (nInternSize < nMinSize)
								{
									entry.m_pHeader = &vecHeadersIntern[nIndexInternal];
									entry.m_strArchiveName = strInternFile;
									nMinSize = nInternSize;
								}

								for (x = 0; x < pApp->g_dwNumExtern; x++)
								{
									if (nExternSize[x] < nMinSize)
									{
										entry.m_pHeader = &vecHeadersExtern[x][nIndexExternal[x]];
										entry.m_strArchiveName = strExternFile[x];
										nMinSize = nExternSize[x];
									}
								}

								// Determine all the ones that have the minimum size.
								cMinStr += "\n    " + strName + "  --> ";
								if (nOldSize == nMinSize)
									cMinStr += " ORG";

								if (nInternSize == nMinSize)
									cMinStr += " INT";

								for (x = 0; x < pApp->g_dwNumExtern; x++)
								{
									if (nExternSize[x] == nMinSize)
									{
										CString cExtNumber;
										cExtNumber.Format(" %03u",x+1);
										cMinStr += cExtNumber;
									}
								}

								vecEntries.push_back(entry);
							}
						}

						if (! bError && (DWORD)vecEntries.size() > 0)
						{
							// sort filenames alphabetically

							std::stable_sort(vecEntries.begin(), vecEntries.end());

							// create the mixed zipfile

							BOOL bOk = BuildNewZip(strSource + strTmpExt, &vecEntries);

							// do an additional zipfiletest

							if (bOk)
								bOk &= TestNewZip(strSource + strTmpExt);

							if (bOk)
							{
								CFileStatus stOld, stNew;
								bOk &= CFile::GetStatus(strSource + strTmpExt, stNew);
								bOk &= CMisc::SetFileWriteable(strSource, &stOld);

								const DWORD nGain = stOld.m_size - stNew.m_size;

								if (bOk)
								{
									// calculate gain

									if (nGain > 0)
									{
										if (pApp->g_bStop)
											break;

										q.LoadString(IDS_UPDATE);
										p->pStatus1->SetWindowText(q + CMisc::GetPureFileName(strSource));

										nTotalGain += nGain;

										// replace old file with new file

										if (! SafeRename(strSource))
										{
											bError = TRUE;
											strError.LoadString(IDS_CANTREPLACE);
										}
									}
								}


								// insert file to list

								if (! bError)
								{
									// reset time/date, set archive flag

									CFileStatus status;
									if (CFile::GetStatus(strSource, status))
									{
										if (! pApp->g_bSetCurrentTime)
										{
											status.m_atime = stOld.m_atime;
											status.m_ctime = stOld.m_ctime;
											status.m_mtime = stOld.m_mtime;
										}
										else
										{
											status.m_ctime = CTime::GetCurrentTime();
											status.m_atime = status.m_ctime;
											status.m_mtime = status.m_ctime;
										}

										status.m_attribute |= CFile::archive;

										try
										{
											CFile::SetStatus(strSource, status);
										}
										catch (CFileException *e)
										{
											e->Delete();
										}
									}

									CString strGain = CMisc::NumericToString(nGain);
									q.LoadString(IDS_OK);
									UpdateList(p->pListCtrl, i, strSource, q, strGain);
									bFileListed = TRUE;
									WriteLog(pFile, strSource + " (" + strGain +")" + cMinStr);
								}
							}
							else
								strError.LoadString(IDS_CANTCREATE);

							// if tempfile still exists (could that happen anyway ?), remove it
							CMisc::RemoveAFile(strSource + strTmpExt);
						}
					}
					else
						strError.LoadString(IDS_EXTERNALFAIL);
				}
				else
					strError.LoadString(IDS_INTERNALFAIL);
			}
			else
				strError.LoadString(IDS_UNZIPFAIL);
		}
		else
			strError.LoadString(IDS_CANTCLEANUP);

		if (! bFileListed)
		{
			// show file as failed

			q.LoadString(IDS_ERROR);
			UpdateList(p->pListCtrl, i, strSource, q, "-");
			WriteLog(pFile, q + ": " + strSource + " -> " + strError + "\n");
		}


		// calculate remaining time, increase progressbarvalue + update statusbar

		// maybe it should better be calculated on the sum of the decompressed file
		// sizes instead of using the size of the zipfiles
		// ...but then you have to get all fileheaders...

		p->nRemainingSize -= nCurrentSize;
		p->nDoneSize += nCurrentSize;
		p->nDoneTime += (DWORD) CMisc::GetTimeSpan(FALSE).GetTotalSeconds();

		p->pProgressCtrl->StepIt();
		const double dRemainingTime = (double)p->nDoneTime * (double)p->nRemainingSize / ((p->nDoneSize == 0) ? (double)1.0 : (double)p->nDoneSize);
		UpdateStatus(p->pStatus2, i + 1, (DWORD)p->pvecFiles->size(), nTotalGain, (DWORD)dRemainingTime, p->nDoneSize);
	}


	// clean up folders

	CMisc::RemoveAFolder(strUnzipFolder);

	for (x = 0; x < pApp->g_dwNumExtern; x++)
		CMisc::RemoveAFolder(strExternFolder[x]);

	CMisc::RemoveAFolder(strInternFolder);


	// show total gain and beep :O)

	p->pProgressCtrl->SetPos(0);

	CString qq;
	qq.LoadString(IDS_TOTAL);
	CString strTotalGain = CMisc::NumericToString(nTotalGain);
	CString strTotal;
	strTotal.Format(qq, strTotalGain);
	p->pStatus1->SetWindowText(strTotal);

	if (! pApp->g_bNoBeep)
	{
		Beep(1000, 100);
		Beep(1500, 100);
		Beep(2000, 100);
	}

	WriteLog(pFile, "\n" + strTotal + "\n--------");


	// close logfile (if any)

	if (pFile != NULL)
	{
		pFile->Flush();
		pFile->Close();
	}


	// clear thread flag and end

	pApp->g_bThreadRunning = FALSE;

	return 0;
}


void CZipMaxDlg::StartZipping ()
{
	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();

	DWORD nListSize = (int)m_vecFiles.size();

	// init statusline, progressbar, etc..

	m_pListCtrl->DeleteAllItems();
	m_pStatus1->SetWindowText("");
	m_pProgCtrl->SetPos(0);
	m_pProgCtrl->SetStep(1);
	m_pProgCtrl->SetRange(0, nListSize);


	// any files at all ? if not, do nothing

	if (nListSize == 0)
		return;


	// get total filesize (only for remaining time calculation)

	UINT64 nTotalSize = 0;
	UINT64 nSize = 0;

	for (DWORD i = 0; i < nListSize; i++)
	{
		CMisc::GetFileSize(m_vecFiles[i], &nSize);
		nTotalSize += nSize;
	}


	// fill thread-Parameter

	m_param.pstrMainFolder = &(pApp->g_strMainFolder);
	m_param.pProgressCtrl = m_pProgCtrl;
	m_param.pListCtrl = m_pListCtrl;
	m_param.pStatus1 = m_pStatus1;
	m_param.pStatus2 = m_pStatus2;
	m_param.pvecFiles = &m_vecFiles;
	m_param.nRemainingSize = nTotalSize;
	m_param.nDoneSize = (UINT64) 0;
	m_param.nDoneTime = (DWORD) 0;


	// reset stopflag

	pApp->g_bStop = FALSE;


	// start thread

	m_pThread = AfxBeginThread(WorkThread, &m_param, GetThreadPriority());

	return;
}

int CALLBACK CZipMaxDlg::CompareProc (LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	LPSTR t1,t2;
	SORTINFO *si;
	CListCtrl *pListCtrl;
	CString s1,s2,cstrDashes;
	int nColumn,nSortDirection,nLen;

	si = (SORTINFO *)lParamSort;
	pListCtrl = si->pListCtrl;
	nColumn = si->nColumn;
	nSortDirection = si->nSortDirection;
	s1 = pListCtrl->GetItemText((int)lParam1,nColumn);
	s2 = pListCtrl->GetItemText((int)lParam2,nColumn);

	// If it is column 2 (which has a number with a thousands separator), then simply
	// sorting the strings does not work (e.g. the strings 123 89 7 would be sorted
	// as 123 7 89). An easy solution is to prepend dashes to the shortest of the
	// pair of strings so that both strings have the same length. We use dashes so
	// that entries that have a dash in this column appear at the top when sorting
	// in ascending order.
	if (nColumn == 2)
	{
		nLen = max(s1.GetLength(),s2.GetLength());
		cstrDashes = CString('-',nLen);
		s1 = (cstrDashes + s1).Right(nLen);
		s2 = (cstrDashes + s2).Right(nLen);
	}

	t1 = (LPSTR)LPCTSTR(s1);
	t2 = (LPSTR)LPCTSTR(s2);

	return nSortDirection * _stricmp(t1,t2);
}

void CZipMaxDlg::GetColumnInfo (int nColumn, LVCOLUMN *pcolumn, CString &str)
{
	switch (nColumn)
	{
		case 0:
			str.LoadString(IDS_FILE);
			pcolumn->fmt = LVCFMT_LEFT;
			pcolumn->pszText = (LPSTR)LPCTSTR(str);
			break;
		case 1:
			str.LoadString(IDS_STATUS);
			pcolumn->fmt = LVCFMT_CENTER;
			pcolumn->pszText = (LPSTR)LPCTSTR(str);
			break;
		case 2:
			str.LoadString(IDS_GAINEDBYTES);
			pcolumn->fmt = LVCFMT_RIGHT;
			pcolumn->pszText = (LPSTR)LPCTSTR(str);
			break;
	}

	return;
}

void CZipMaxDlg::OnColumnClick (NMHDR *pNMHDR, LRESULT *pResult)
{
	SORTINFO si;
	CString str;
	LVCOLUMN column;
	CZipMaxApp *pApp;
	NMLISTVIEW *pNMListView;
	int i,nNumInList,nColumn;

	// Don't try sorting columns while busy or when the list is empty.
	pApp = (CZipMaxApp *)AfxGetApp();
	nNumInList = m_pListCtrl->GetItemCount();
	if ((pApp->g_bThreadRunning) || (nNumInList == 0))
	{
		*pResult = FALSE;
		return;
	}

	pNMListView = (NMLISTVIEW *)pNMHDR;
	nColumn = pNMListView->iSubItem;
	m_nSortDirection = (nColumn == m_nLastColumn) ? -m_nSortDirection : 1;

	// lParam1 and lParam2 in CompareProc() give the lParam of the respective items,
	// so we make sure that the lParam of each item is its index in the current list.
	for (i = 0; i < nNumInList; i++)
		m_pListCtrl->SetItem(i,0,LVIF_PARAM,NULL,0,0,0,i);

	si.pListCtrl = m_pListCtrl;
	si.nColumn = nColumn;
	si.nSortDirection = m_nSortDirection;
	m_pListCtrl->SortItems(CompareProc,(LPARAM)&si);

	if ((m_nLastColumn >= 0) && (nColumn != m_nLastColumn))
	{
		GetColumnInfo(m_nLastColumn,&column,str);
		column.mask = LVCF_FMT;
		m_pListCtrl->SetColumn(m_nLastColumn,&column);
	}

	GetColumnInfo(nColumn,&column,str);
	column.mask = LVCF_FMT | LVCF_IMAGE;
	column.iImage = (m_nSortDirection == 1) ? 3 : 4;
	column.fmt |= LVCFMT_IMAGE | LVCFMT_COL_HAS_IMAGES | LVCFMT_BITMAP_ON_RIGHT;
	m_pListCtrl->SetColumn(nColumn,&column);

	m_nLastColumn = nColumn;

	*pResult = TRUE;

	return;
}

void CZipMaxDlg::OnBnClickedButtonPause()
{
	CZipMaxApp *pApp = (CZipMaxApp *)AfxGetApp();

	if (pApp->g_pi.hThread == 0)
		return;

	pApp->g_bPause = ! pApp->g_bPause;

	if (pApp->g_bPause)
		SuspendThread(pApp->g_pi.hThread);
	else
		ResumeThread(pApp->g_pi.hThread);

	if (m_pStatus1 != NULL && IsWindow(m_pStatus1->m_hWnd))
	{
		if (pApp->g_bPause)
		{
			CString s;
			s.LoadString(IDS_PAUSETEXT);
			m_pStatus1->GetWindowText(pApp->g_strOldStatus1);
			m_pStatus1->SetWindowText(s);
		}
		else
			m_pStatus1->SetWindowText(pApp->g_strOldStatus1);

		m_pStatus1->Invalidate();
	}

	return;
}
